@@ -3,17 +3,33 @@ import collections
 
 
 class Channel:
-    """A simple asynchronous channel for sending and receiving messages."""
+    """A simple asynchronous channel for sending and receiving messages.
+
+    This class represents a channel that allows sending and receiving messages asynchronously.
+    Messages can be sent using the `send` method, and received using the `recv` method.
+    If the channel is empty, the `recv` method will wait until a message is sent.
+
+    """
 
     def __init__(self, loop: asyncio.AbstractEventLoop):
-        """Initialize the channel with empty queue and waiters."""
+        """Initialize the channel with empty queue and waiters.
+
+        Args:
+            loop (asyncio.AbstractEventLoop): The event loop to associate with the channel.
+
+        """
         self._loop = loop
 
         self._queue = collections.deque()
         self._waiters = collections.deque()
 
     async def send(self, message):
-        """Send a message to the channel. If there are any waiters, wake one up."""
+        """Send a message to the channel. If there are any waiters, wake one up.
+
+        Args:
+            message: The message to send.
+
+        """
         self._queue.append(message)
         if self._waiters:
             waiter = self._waiters.popleft()
@@ -21,7 +37,12 @@ class Channel:
                 waiter.set_result(None)
 
     async def recv(self):
-        """Receive a message from the channel. If the channel is empty, wait until a message is sent."""
+        """Receive a message from the channel. If the channel is empty, wait until a message is sent.
+
+        Returns:
+            The received message.
+
+        """
         while not self._queue:
             waiter = self._loop.create_future()
             self._waiters.append(waiter)
