@@ -4,11 +4,13 @@ import uuid
 from . import config
 from .broker import Broker
 from .debug import debug
-from .errors import print_exception
 from .gossip import Gossip
 from .members import Members
 from .message_pb2 import Message
+from .task_manager import TaskManager
+from .topology import Node
 from .transport import Transport
+from .transport.address import parse_addr
 
 
 class Peer:
@@ -27,15 +29,12 @@ class Peer:
         else:
             self.peer_id = uuid.uuid1().bytes
 
-        # FIXME: should be lazy
         self.transport = Transport((host, port), loop=self._loop)
         self.gossip = Gossip(self.transport, fanout=fanout, peer_id=self.peer_id)
         self.broker = Broker(self.gossip, loop=self._loop)
 
-        self.task = self._loop.create_task(self.broker.listen())
-        self.task.add_done_callback(print_exception)
-
-        self.tasks = []
+        self.task_manager = TaskManager(loop=self._loop)
+        self.task_manager.create_task(self.broker.listen())
 
         self.members = Members(self)
 
@@ -49,12 +48,12 @@ class Peer:
 
     @property
     def DSN(self):
-        return "{}@{}:{}".format(self.node["node_id"].decode(), *self.node["node_addr"])
+        return "{}@{}:{}".format(self.node.node_id.decode(), *self.node.node_addr)
 
     async def _connect(self):
         await self.gossip.send_gossip_handshake()
 
-    def connect(self, seeds=config.GOSSIP_SEEDS):
+    def connect(self, seeds=config.SEEDS):
         if not seeds:
             return
 
@@ -64,28 +63,23 @@ class Peer:
             seeds = seeds.split(",")
             for seed in seeds:
                 peer_id, addr = seed.split("@")
-                host, port = addr.split(":")
-                nodes.append({"node_id": peer_id.encode(), "node_addr": (host, int(port))})
+                addr = parse_addr(addr)
+                nodes.append(Node(node_id=peer_id.encode(), node_addr=addr))
 
         elif isinstance(seeds, list):
             nodes = seeds
 
         self.gossip.topology.add(nodes)
-        task = self._loop.create_task(self._connect())
-        task.add_done_callback(print_exception)
-        self.tasks.append(task)
+        self.task_manager.create_task(self._connect())
 
     async def disconnect(self):
+        await self.members.close()
         await self.broker.close()
-
-        for task in self.tasks:
-            task.cancel()
-            await asyncio.gather(task, return_exceptions=True)
-
-        self.task.cancel()
-        await asyncio.gather(self.task, return_exceptions=True)
+        await self.task_manager.close()
 
     async def publish(self, topic, message, peers=None, syn=False):
+        if not message.id:
+            message.id = uuid.uuid4().bytes
         topic = topic.replace("{uuid}", uuid.uuid4().hex)
 
         if syn and (Message.Kind.SYN not in message.kind):
@@ -135,5 +129,5 @@ class Peer:
         finally:
             if main:
                 main.cancel()
-            self._loop.run_until_complete(main)
+                self._loop.run_until_complete(main)
             self._loop.run_until_complete(self.disconnect())
